from typing import Any

from ..database import db
from ..database.types import Serial


class Plugin(db.Model):
    __tablename__ = "plugin"
    id: int = db.Column("id", Serial, primary_key=True)
    name: str = db.Column(db.String(127), nullable=False)
    version: str = db.Column(db.String(30), nullable=False)
    """The latest installed version"""
    enabled: bool = db.Column(db.Boolean, default=False)

    settings_ = db.relationship("PluginSetting", cascade="all, delete")
    permissions_ = db.relationship("Permission", cascade="all, delete")


class PluginSetting(db.Model):
    __tablename__ = "plugin_setting"
    id = db.Column("id", Serial, primary_key=True)
    plugin_id: int = db.Column("plugin", Serial, db.ForeignKey("plugin.id"))
    name: str = db.Column(db.String(127), nullable=False)
    value: Any = db.Column(db.PickleType(protocol=4))
